@@ -170,6 +170,10 @@ gda_tree_class_init (GdaTreeClass *klass)
         object_class->set_property = gda_tree_set_property;
         object_class->get_property = gda_tree_get_property;
 
+	/**
+	 * GdaTree:is-list:
+	 * Tells if the GdaTree is a list or a tree.
+	 */
 	g_object_class_install_property (object_class, PROP_IS_LIST,
                                          g_param_spec_boolean ("is-list", _("Tells if the GdaTree is a list or a tree"), NULL,
 							       FALSE, G_PARAM_READABLE));
@@ -178,7 +182,7 @@ gda_tree_class_init (GdaTreeClass *klass)
 }
 
 static void
-gda_tree_init (GdaTree *tree, GdaTreeClass *klass)
+gda_tree_init (GdaTree *tree, G_GNUC_UNUSED GdaTreeClass *klass)
 {
 	g_return_if_fail (GDA_IS_TREE (tree));
 
@@ -247,7 +251,8 @@ gda_tree_get_type (void)
                         NULL,
                         sizeof (GdaTree),
                         0,
-                        (GInstanceInitFunc) gda_tree_init
+                        (GInstanceInitFunc) gda_tree_init,
+			0
                 };
 
                 g_static_mutex_lock (&registering);
@@ -261,7 +266,7 @@ gda_tree_get_type (void)
 static void
 gda_tree_set_property (GObject *object,
 			 guint param_id,
-			 const GValue *value,
+			 G_GNUC_UNUSED const GValue *value,
 			 GParamSpec *pspec)
 {
 	GdaTree *tree;
@@ -269,6 +274,9 @@ gda_tree_set_property (GObject *object,
         tree = GDA_TREE (object);
         if (tree->priv) {
                 switch (param_id) {
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}	
 	}
 }
@@ -296,6 +304,9 @@ gda_tree_get_property (GObject *object,
 			g_value_set_boolean (value, is_list);
 			break;
 		}
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}	
 }
@@ -537,7 +548,7 @@ real_gda_tree_get_nodes_in_path (GdaTree *tree, GSList *segments, gboolean use_n
 		if (use_names)
 			node = gda_tree_node_get_child_name (parent, (gchar *) seglist->data);
 		else
-			node = gda_tree_node_get_child_index (parent, atoi ((gchar *) seglist->data));
+			node = gda_tree_node_get_child_index (parent, atoi ((gchar *) seglist->data)); /* Flawfinder: ignore */
 		if (!node && tree->priv->update_on_searching) {
 			/* update level if necessary */
 			mgrlist = _gda_tree_node_get_managers_for_children (parent);
@@ -551,7 +562,7 @@ real_gda_tree_get_nodes_in_path (GdaTree *tree, GSList *segments, gboolean use_n
 			if (use_names)
 				node = gda_tree_node_get_child_name (parent, (gchar *) seglist->data);
 			else
-				node = gda_tree_node_get_child_index (parent, atoi ((gchar *) seglist->data));
+				node = gda_tree_node_get_child_index (parent, atoi ((gchar *) seglist->data)); /* Flawfinder: ignore */
 		}
 		if (!node) 
 			return NULL;
@@ -600,7 +611,7 @@ gda_tree_get_node_path (GdaTree *tree, GdaTreeNode *node)
 	if (build_node_path (tree, node, array))
 		str = g_strjoinv (":", (gchar **) array->data);
 	
-	gint i;
+	gsize i;
 	for (i = 0; i < array->len; i++)
 		g_free (g_array_index (array, gchar *, i));
 	g_array_free (array, TRUE);
@@ -636,7 +647,7 @@ build_node_path (GdaTree *tree, GdaTreeNode *node, GArray *array)
 }
 
 /**
- * gda_tree_get_node
+ * gda_tree_get_node:
  * @tree: a #GdaTree object
  * @tree_path: full path to the required nodes (if @use_names is %TRUE, then it must start with '/')
  * @use_names: if %TRUE, then @tree_path will be interpreted as a unix style path, and if %FALSE,
@@ -644,7 +655,7 @@ build_node_path (GdaTree *tree, GdaTreeNode *node, GArray *array)
  *
  * Locates a #GdaTreeNode using the @tree_path path.
  *
- * Returns: (transfer none): the requested #GdaTreeNode pointer, or %NULL if not found
+ * Returns: (transfer none) (allow-none): the requested #GdaTreeNode pointer, or %NULL if not found
  *
  * Since: 4.2
  */
@@ -929,7 +940,7 @@ node_has_child_toggled_cb (GdaTreeNode *reporting, GdaTreeNode *node, GdaTree *t
 }
 
 static void
-node_deleted_cb (GdaTreeNode *reporting, const gchar *relative_path, GdaTree *tree)
+node_deleted_cb (G_GNUC_UNUSED GdaTreeNode *reporting, const gchar *relative_path, GdaTree *tree)
 {
 	g_signal_emit (tree, gda_tree_signals [NODE_DELETED], 0, relative_path);
 }
