@@ -10,6 +10,7 @@
 
 #define CHECK_EXTRA_INFO
 /*#undef CHECK_EXTRA_INFO*/
+static gboolean do_test_signal (void);
 static gboolean do_test_common_read (GdaDataModel *proxy);
 static gboolean do_test_read_direct_1 (void);
 static gboolean do_test_read_direct_2 (void);
@@ -20,6 +21,8 @@ static gboolean do_test_array (void);
 static gboolean do_test_prop_change (void);
 static gboolean do_test_proxied_model_modif (void);
 
+static gboolean do_test_delete_rows (void);
+
 /* 
  * the tests are all run with all the possible combinations of defer_sync (TRUE/FALSE) and 
  * prepend_null_row (TRUE/FALSE), so the ADJUST_ROW() macro allows for row adjustment
@@ -52,58 +55,80 @@ main (int argc, char **argv)
 {
 	int number_failed = 0;
 
-	gda_init ("check-data-proxy", PACKAGE_VERSION, argc, argv);
+	/* set up test environment */
+        g_setenv ("GDA_TOP_BUILD_DIR", TOP_BUILD_DIR, 0);
+        g_setenv ("GDA_TOP_SRC_DIR", TOP_SRC_DIR, TRUE);
+	gda_init ();
 
 	prepend_null_row = FALSE;
-	defer_sync  = FALSE;
+ 	defer_sync = FALSE;
+	if (!do_test_delete_rows ())
+		number_failed ++;
+	defer_sync = TRUE;
+	if (!do_test_delete_rows ())
+		number_failed ++;
+
+	prepend_null_row = TRUE;
+ 	defer_sync = FALSE;
+	if (!do_test_delete_rows ())
+		number_failed ++;
+	defer_sync = TRUE;
+	if (!do_test_delete_rows ())
+		number_failed ++;
+	
+	if (!do_test_signal ())
+		number_failed ++;
+
+	prepend_null_row = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_read_direct_1 ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_read_direct_1 ())
 		number_failed ++;
 
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_read_direct_2 ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_read_direct_2 ())
 		number_failed ++;
 
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_array ())
 		number_failed ++;
 
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_array ())
 		number_failed ++;
 
 	prepend_null_row = TRUE;
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_read_direct_1 ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_read_direct_1 ())
 		number_failed ++;
 
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_read_direct_2 ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_read_direct_2 ())
 		number_failed ++;
 
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_array ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_array ())
 		number_failed ++;
 
 	prepend_null_row = FALSE;
-	defer_sync  = FALSE;
+	defer_sync = FALSE;
 	if (!do_test_prop_change ())
 		number_failed ++;
-	defer_sync  = TRUE;
+	defer_sync = TRUE;
 	if (!do_test_prop_change ())
 		number_failed ++;
 
@@ -130,6 +155,117 @@ main (int argc, char **argv)
 	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
+static GError *
+validate_row_changes (GdaDataProxy *proxy, gint row, gint proxied_row, gchar *token)
+{
+	const GValue *cvalue;
+	/* refuse if population < 100 */
+	
+	cvalue = gda_data_model_get_value_at ((GdaDataModel*) proxy, 2, row, NULL);
+	g_assert (cvalue);
+	if (G_VALUE_TYPE (cvalue) == GDA_TYPE_NULL)
+		return NULL;
+	else {
+		gint pop;
+		pop = g_value_get_int (cvalue);
+		if (pop < 100) {
+			GError *error = NULL;
+			g_set_error (&error, 0, 0,
+				     "%s", "Population is too small");
+			return error;
+		}
+		else
+			return NULL;
+	}
+}
+
+static gboolean
+do_test_signal (void)
+{
+#define FILE "city.csv"
+	gchar *file;
+	GdaDataModel *import, *proxy, *rw_model;
+	GSList *errors;
+	GdaSet *options;
+	GValue *value;
+	GError *lerror = NULL;
+
+	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
+	options = gda_set_new_inline (2, 
+				      "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE,
+				      "G_TYPE_2", G_TYPE_GTYPE, G_TYPE_INT);
+	import = gda_data_model_import_new_file (file, TRUE, options);
+	g_free (file);
+	g_object_unref (options);
+
+	if ((errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (import)))) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not load file '%s'\n", FILE);
+#endif
+		g_object_unref (import);
+		return FALSE;
+	}
+
+	rw_model = (GdaDataModel*) gda_data_model_array_copy_model (import, NULL);
+	g_assert (rw_model);
+	g_object_unref (import);
+	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", rw_model, "sample-size", 0, NULL);
+	if (!proxy) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not create GdaDataProxy\n");
+#endif
+		return FALSE;
+	}
+
+	g_signal_connect (G_OBJECT (proxy), "validate_row_changes",
+			  G_CALLBACK (validate_row_changes), "Token");
+
+	/**/
+	g_value_set_int ((value = gda_value_new (G_TYPE_INT)), 23);
+	g_assert (gda_data_model_set_value_at (proxy, 2, 5, value, NULL));
+	g_value_set_int (value, 100);
+	g_assert (gda_data_model_set_value_at (proxy, 2, 3, value, NULL));
+	gda_value_free (value);
+
+	if (gda_data_proxy_apply_all_changes (GDA_DATA_PROXY (proxy), &lerror)) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: gda_data_proxy_apply_all_changes() should have failed\n");
+#endif
+		return FALSE; 
+	}
+	if (!lerror) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: returned error should not be NULL\n");
+#endif
+		return FALSE; 
+	}
+	if (strcmp (lerror->message, "Population is too small")) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: returned error message should be 'Population is too small', and got '%s'\n", 
+			 lerror->message);
+#endif
+		return FALSE; 
+	}
+	g_error_free (lerror);
+
+	/**/
+	g_assert (gda_data_proxy_cancel_all_changes (GDA_DATA_PROXY (proxy)));
+	g_value_set_int ((value = gda_value_new (G_TYPE_INT)), 123);
+	g_assert (gda_data_model_set_value_at (proxy, 2, 5, value, NULL));
+	gda_value_free (value);
+	if (!gda_data_proxy_apply_all_changes (GDA_DATA_PROXY (proxy), &lerror)) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: gda_data_proxy_apply_all_changes() should not have failed, error: %s\n",
+			 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+		return FALSE; 
+	}
+
+	g_object_unref (proxy);
+	g_object_unref (rw_model);
+
+	return TRUE;
+}
 
 static gboolean
 do_test_read_direct_1 ()
@@ -154,7 +290,7 @@ do_test_read_direct_1 ()
 		return FALSE;
 	}
 
-	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", import, "sample_size", 0, NULL);
+	proxy = g_object_new (GDA_TYPE_DATA_PROXY, "model", import, "sample-size", 0, NULL);
 	g_object_unref (import);
 	if (!proxy) {
 #ifdef CHECK_EXTRA_INFO
@@ -162,7 +298,7 @@ do_test_read_direct_1 ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_read (proxy);
 }
 
@@ -197,7 +333,7 @@ do_test_read_direct_2 ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_read (proxy);
 }
 
@@ -244,7 +380,7 @@ do_test_array ()
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	return do_test_common_write (proxy);
 }
 
@@ -291,19 +427,19 @@ do_test_prop_change (void)
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 0);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, NULL);
 
 	gboolean retval = FALSE;
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/*
@@ -389,20 +525,20 @@ do_test_proxied_model_modif (void)
 #endif
 		return FALSE;
 	}
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 0);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 
 	gboolean retval = FALSE;
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	
@@ -433,7 +569,7 @@ do_test_proxied_model_modif (void)
 	clean_expected_signals (proxy);
 	if (!check_data_model_n_rows (proxy, 159)) goto out;
 
-	values = make_values_list (0, G_TYPE_STRING, "BigCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "567890", 0);
+	values = make_values_list (0, G_TYPE_STRING, "BigCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "567890", (GType) 0);
 	declare_expected_signals ("U158", "Set values of new proxied row - 1");
 	if (!check_data_model_set_values (model, 158, values)) goto out;
 	free_values_list (values);
@@ -499,16 +635,16 @@ do_test_proxied_model_modif (void)
 	/*
 	 * change new proxied's rows values
 	 */
-	values = make_values_list (0, G_TYPE_STRING, "SmallCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "4907", 0);
+	values = make_values_list (0, G_TYPE_STRING, "SmallCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "4907", (GType) 0);
 	declare_expected_signals ("U1", "Set values of new proxied row - 2");
 	if (!check_data_model_set_values (model, 21, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
 	if (!check_data_model_value (proxy, 1, 0, G_TYPE_STRING, "SmallCity2")) goto out;
-	if (!check_data_model_value (proxy, 1, 1, G_TYPE_STRING, NULL)) goto out;
+	if (!check_data_model_value (proxy, 1, 1, G_TYPE_STRING, "ARG")) goto out;
 	if (!check_data_model_value (proxy, 1, 2, G_TYPE_STRING, "4907")) goto out;
 	if (!check_data_model_value (proxy, 1, 3, G_TYPE_STRING, "SmallCity2")) goto out;
-	if (!check_data_model_value (proxy, 1, 4, G_TYPE_STRING, NULL)) goto out;
+	if (!check_data_model_value (proxy, 1, 4, G_TYPE_STRING, "ARG")) goto out;
 	if (!check_data_model_value (proxy, 1, 5, G_TYPE_STRING, "4907")) goto out;
 
 	/* 
@@ -524,7 +660,7 @@ do_test_proxied_model_modif (void)
 	/* 
 	 * delete a proxy row, in current proxy's chunk
 	 */
-	declare_expected_signals ("R5", "Delete a proxy row - 3");
+	declare_expected_signals ("R6", "Delete a proxy row - 3");
 	if (!check_data_model_remove_row (model, 25)) goto out;
 	clean_expected_signals (proxy);
 	gda_data_model_dump (proxy, stdout);
@@ -543,16 +679,16 @@ do_test_common_read (GdaDataModel *proxy)
 	gboolean retval = FALSE;
 
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 10);
-	g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, NULL);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/* 
@@ -583,7 +719,7 @@ do_test_common_read (GdaDataModel *proxy)
 
 
 	filter = gda_data_proxy_get_filter_expr (GDA_DATA_PROXY (proxy));
-	if (!filter || strcmp (filter, "countrycode = 'BFA'")) {
+	if (!filter || strcmp (filter, "(countrycode = 'BFA')")) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Filter should be '%s' and is '%s'\n", "countrycode = 'BFA'", filter);
 #endif
@@ -665,16 +801,16 @@ do_test_common_write (GdaDataModel *proxy)
 	pmodel = gda_data_proxy_get_proxied_model (GDA_DATA_PROXY (proxy));
 
 	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 10);
-	g_object_set (G_OBJECT (proxy), "defer_sync", defer_sync, 
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
 		      "prepend-null-entry", prepend_null_row, NULL);
 
 	g_signal_connect (G_OBJECT (proxy), "reset",
 			  G_CALLBACK (proxy_reset_cb), NULL);
-	g_signal_connect (G_OBJECT (proxy), "row_inserted",
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
 			  G_CALLBACK (proxy_row_cb), "I");
-	g_signal_connect (G_OBJECT (proxy), "row_updated",
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
 			  G_CALLBACK (proxy_row_cb), "U");
-	g_signal_connect (G_OBJECT (proxy), "row_removed",
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
 			  G_CALLBACK (proxy_row_cb), "R");
 
 	/*
@@ -728,7 +864,7 @@ do_test_common_write (GdaDataModel *proxy)
 	/*
 	 * set several values for the new row 
 	 */
-	values = make_values_list (0, G_TYPE_STRING, "MyCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "12345", 0);
+	values = make_values_list (0, G_TYPE_STRING, "MyCity2", G_TYPE_STRING, NULL, G_TYPE_STRING, "12345", (GType) 0);
 	declare_expected_signals ("U159", "Set values of new row - new row");
 	if (!check_data_model_set_values (proxy, 159, values)) goto out;
 	free_values_list (values);
@@ -741,7 +877,7 @@ do_test_common_write (GdaDataModel *proxy)
 	if (!check_data_model_value (proxy, 159, 5, G_TYPE_STRING, NULL)) goto out;
 	if (!check_data_model_n_rows (proxy, 160)) goto out;
 
-	values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", 0);
+	values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", (GType) 0);
 	declare_expected_signals ("U157", "Set values of new row - existing row");
 	if (!check_data_model_set_values (proxy, 157, values)) goto out;
 	free_values_list (values);
@@ -838,16 +974,16 @@ do_test_common_write (GdaDataModel *proxy)
 	 * Set values for those 3 new rows
 	 */
 	declare_expected_signals ("4U50", "Set values of 3 new rows");
-	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", (GType) 0);
 	if (!check_data_model_set_values (proxy, 50, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", (GType) 0);
 	if (!check_data_model_set_values (proxy, 51, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", (GType) 0);
 	if (!check_data_model_set_values (proxy, 52, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", (GType) 0);
 	if (!check_data_model_set_values (proxy, 53, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
@@ -944,16 +1080,16 @@ do_test_common_write (GdaDataModel *proxy)
 	 * Set values for those 4 new rows
 	 */
 	declare_expected_signals ("4U159", "Set values of 4 new rows");
-	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR1", G_TYPE_STRING, "AAA", G_TYPE_STRING, "319", (GType) 0);
 	if (!check_data_model_set_values (proxy, 159, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR2", G_TYPE_STRING, "BBB", G_TYPE_STRING, "320", (GType) 0);
 	if (!check_data_model_set_values (proxy, 160, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR3", G_TYPE_STRING, "CCC", G_TYPE_STRING, "321", (GType) 0);
 	if (!check_data_model_set_values (proxy, 161, values)) goto out;
 	free_values_list (values);
-	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", 0);
+	values = make_values_list (0, G_TYPE_STRING, "NR4", G_TYPE_STRING, "DDD", G_TYPE_STRING, "330", (GType) 0);
 	if (!check_data_model_set_values (proxy, 162, values)) goto out;
 	free_values_list (values);
 	clean_expected_signals (proxy);
@@ -1045,7 +1181,7 @@ do_test_common_write (GdaDataModel *proxy)
 		/*
 		 * Try to alter first row
 		 */
-		values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", 0);
+		values = make_values_list (0, G_TYPE_STRING, "MyCity3", G_TYPE_STRING, "ZZZ", G_TYPE_STRING, "787", (GType) 0);
 		if (gda_data_model_set_values (proxy, 0, values, NULL)) {
 #ifdef CHECK_EXTRA_INFO
 			g_print ("ERROR: Should not be able to alter row 0 (as it is the prepended row)\n");
@@ -1124,6 +1260,8 @@ gint         expected_signals_index;
  *   - 2U123 means 2 updates, 1 for row 123 and 1 for row 124
  *   - 2I4 means 2 inserts, 1 for row 4 and 1 for row 5
  *   - 2R6 means 2 deletes, all for row 6
+ *
+ * Use lower case to avoid taking into account the prepend_null_row variable
  */
 static void
 declare_expected_signals (const gchar *expected, const gchar *name)
@@ -1159,6 +1297,11 @@ declare_expected_signals (const gchar *expected, const gchar *name)
 			ptr++;
 		for (i = 0; i < nb; i++) {
 			gint sigrow;
+			gboolean use_prepend_null_row = TRUE;
+			if (type >= 'a') {
+				type += 'A' - 'a';
+				use_prepend_null_row = FALSE;
+			}
 			switch (type) {
 			case 'U':
 			case 'I':
@@ -1173,7 +1316,10 @@ declare_expected_signals (const gchar *expected, const gchar *name)
 
 			ASignal sig;
 			sig.sig = type;
-			sig.row = ADJUST_ROW (sigrow);
+			if (use_prepend_null_row)
+				sig.row = ADJUST_ROW (sigrow);
+			else
+				sig.row = sigrow;
 			g_array_append_val (expected_signals, sig);
 		}
 	}
@@ -1212,8 +1358,16 @@ clean_expected_signals (GdaDataModel *model)
 		}
 		ncols = gda_data_model_get_n_columns (model);
 		for (col = 0; col < ncols; col++) {
-			const GValue *value = gda_data_model_get_value_at (model, col, 0);
-			if (value) {
+			GError *lerror = NULL;
+			const GValue *value = gda_data_model_get_value_at (model, col, 0, &lerror);
+			if (!value) {
+#ifdef CHECK_EXTRA_INFO
+				g_print ("Can't get data model's value: %s\n",
+					 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+				exit (1);
+			}
+			else if (! gda_value_is_null (value)) {
 #ifdef CHECK_EXTRA_INFO
 				g_print ("ERROR: As the \"prepend-null-entry\" property is set, there should always be a first "
 					 "all-NULL entry, got '%s' at column %d\n", gda_value_stringify (value), col);
@@ -1322,6 +1476,7 @@ check_data_model_value (GdaDataModel *model, gint row, gint col, GType type, con
 	GValue *gvalue;
 	const GValue *mvalue;
 	gboolean retval;
+	GError *lerror = NULL;
 
 	row = ADJUST_ROW_FOR_MODEL (model, row);
 
@@ -1335,8 +1490,15 @@ check_data_model_value (GdaDataModel *model, gint row, gint col, GType type, con
 	else
 		gvalue = gda_value_new_null ();	
 
-	mvalue = gda_data_model_get_value_at (model, col, row); 
-	retval = !gda_value_compare_ext (gvalue, mvalue);
+	mvalue = gda_data_model_get_value_at (model, col, row, &lerror);
+	if (!mvalue) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("Can't get data model's value: %s\n",
+			 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+		return FALSE;
+	}
+	retval = !gda_value_compare (gvalue, mvalue);
 	gda_value_free (gvalue);
 
 #ifdef CHECK_EXTRA_INFO
@@ -1366,7 +1528,7 @@ check_data_model_n_rows (GdaDataModel *model, gint nrows)
 		return FALSE;
 	}
 
-	value = gda_data_model_get_value_at (model, 0, nrows);
+	value = gda_data_model_get_value_at (model, 0, nrows, NULL);
 	if (value) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("ERROR: Got a value at row %d, should not get any as it is out of bounds\n", nrows);
@@ -1443,8 +1605,16 @@ check_data_model_set_values (GdaDataModel *model, gint row, GList *values)
 	GList *list;
 	gint col;
 	for (col = 0, list = values; list; col++, list = list->next) {
-		const GValue *cvalue = gda_data_model_get_value_at (model, col, row);
-		if (gda_value_compare_ext (cvalue, (GValue *) list->data)) {
+		GError *lerror = NULL;
+		const GValue *cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
+		if (!cvalue) {
+#ifdef CHECK_EXTRA_INFO
+			g_print ("Can't get data model's value: %s\n",
+				 lerror && lerror->message ? lerror->message : "No detail");
+#endif
+			return FALSE;
+		}
+		if (list->data && gda_value_compare (cvalue, (GValue *) list->data)) {
 #ifdef CHECK_EXTRA_INFO
 			g_print ("ERROR: Read value is not equal to set value: got '%s' and expected '%s'\n",
 				 gda_value_stringify (cvalue), gda_value_stringify ((GValue *) list->data));
@@ -1473,3 +1643,128 @@ check_data_model_remove_row (GdaDataModel *model, gint row)
 
 	return TRUE;
 }
+
+/* remove several rows */
+static gboolean do_test_delete_rows (void)
+{
+#define FILE "city.csv"
+	GError *error = NULL;
+	gchar *file;
+	GdaDataModel *import, *model, *proxy;
+	GSList *errors;
+	GdaSet *options;
+
+	file = g_build_filename (CHECK_FILES, "tests", "data-models", FILE, NULL);
+	options = gda_set_new_inline (1, "TITLE_AS_FIRST_LINE", G_TYPE_BOOLEAN, TRUE);
+	import = gda_data_model_import_new_file (file, TRUE, options);
+	g_free (file);
+	g_object_unref (options);
+
+	if ((errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (import)))) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not load file '%s'\n", FILE);
+#endif
+		g_object_unref (import);
+		return FALSE;
+	}
+
+	model = (GdaDataModel*) gda_data_model_array_copy_model (import, &error);
+	if (!model) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not copy GdaDataModelImport into a GdaDataModelArray: %s\n", 
+			 error && error->message ? error->message : "No detail");
+#endif
+		g_error_free (error);
+		return FALSE;
+	}
+	g_object_unref (import);
+
+	proxy = (GdaDataModel *) gda_data_proxy_new (model);
+	if (!proxy) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not create GdaDataProxy\n");
+#endif
+		return FALSE;
+	}
+	g_object_set (G_OBJECT (proxy), "defer-sync", FALSE, NULL);
+	gda_data_proxy_set_sample_size (GDA_DATA_PROXY (proxy), 0);
+	g_object_set (G_OBJECT (proxy), "defer-sync", defer_sync, 
+		      "prepend-null-entry", prepend_null_row, NULL);
+
+	gboolean retval = FALSE;
+
+	g_signal_connect (G_OBJECT (proxy), "reset",
+			  G_CALLBACK (proxy_reset_cb), NULL);
+	g_signal_connect (G_OBJECT (proxy), "row-inserted",
+			  G_CALLBACK (proxy_row_cb), "I");
+	g_signal_connect (G_OBJECT (proxy), "row-updated",
+			  G_CALLBACK (proxy_row_cb), "U");
+	g_signal_connect (G_OBJECT (proxy), "row-removed",
+			  G_CALLBACK (proxy_row_cb), "R");
+
+	
+	/* 
+	 * mark a proxy row to be deleted
+	 */
+	declare_expected_signals ("u3", "Prepare to delete row 3");
+	gda_data_proxy_delete (GDA_DATA_PROXY (proxy), 3);
+	clean_expected_signals (proxy);
+	if (!check_data_model_n_rows (proxy, 158)) goto out;
+
+	/* 
+	 * mark a proxy row to be deleted
+	 */
+	declare_expected_signals ("u5", "Prepare to delete row 5");
+	gda_data_proxy_delete (GDA_DATA_PROXY (proxy), 5);
+	clean_expected_signals (proxy);
+	if (!check_data_model_n_rows (proxy, 158)) goto out;
+
+	/* 
+	 * mark a proxy row to be deleted
+	 */
+	declare_expected_signals ("u1", "Prepare to delete row 1");
+	gda_data_proxy_delete (GDA_DATA_PROXY (proxy), 1);
+	clean_expected_signals (proxy);
+	if (!check_data_model_n_rows (proxy, 158)) goto out;
+
+	if (!check_data_model_value (proxy, 12, 0, G_TYPE_STRING, "Tirana")) goto out;
+	if (!check_data_model_value (proxy, 12, 2, G_TYPE_STRING, "270000")) goto out;
+	if (!check_data_model_value (proxy, 12, 3, G_TYPE_STRING, "Tirana")) goto out;
+	if (!check_data_model_value (proxy, 12, 5, G_TYPE_STRING, "270000")) goto out;
+
+	/* commit changes */
+	declare_expected_signals ("r1r4r2", "Commit row changes");
+
+	if (! gda_data_proxy_apply_all_changes (GDA_DATA_PROXY (proxy), &error)) {
+#ifdef CHECK_EXTRA_INFO
+		g_print ("ERROR: Could not copy GdaDataModelImport into a GdaDataModelArray: %s\n", 
+			 error && error->message ? error->message : "No detail");
+#endif
+		g_error_free (error);
+		return FALSE;
+	}
+
+	clean_expected_signals (proxy);
+	if (!check_data_model_n_rows (proxy, 155)) goto out;
+
+	if (!check_data_model_value (model, 9, 0, G_TYPE_STRING, "Tirana")) goto out;
+	if (!check_data_model_value (model, 9, 2, G_TYPE_STRING, "270000")) goto out;
+	if (prepend_null_row) {
+		if (!check_data_model_value (model, 0, 0, G_TYPE_STRING, "Herat")) goto out;
+		if (!check_data_model_value (model, 1, 0, G_TYPE_STRING, "Mazar-e-Sharif")) goto out;
+		if (!check_data_model_value (model, 2, 0, G_TYPE_STRING, "Benguela")) goto out;
+	}
+	else {
+		if (!check_data_model_value (model, 0, 0, G_TYPE_STRING, "Oranjestad")) goto out;
+		if (!check_data_model_value (model, 1, 0, G_TYPE_STRING, "Kabul")) goto out;
+		if (!check_data_model_value (model, 2, 0, G_TYPE_STRING, "Qandahar")) goto out;
+	}
+	if (!check_data_model_value (model, 3, 0, G_TYPE_STRING, "Huambo")) goto out;
+	if (!check_data_model_value (model, 4, 0, G_TYPE_STRING, "Lobito")) goto out;
+
+	retval = TRUE;
+ out:
+	g_object_unref (model);
+	g_object_unref (proxy);
+	return retval;
+}
