@@ -108,7 +108,7 @@ meta_changed_cb (GdaMetaStore *store, GSList *changes, gpointer data)
 	}
 	if (expected_changes) {
 		/* expected more changes */
-		gchar *estr = (gchar *) el->data;
+		gchar *estr = (gchar *) expected_changes->data;
 		g_print ("Received no change but EXPECTED GdaMetaStoreChange: %s", estr);
 		exit (EXIT_FAILURE);
 	}
@@ -303,7 +303,9 @@ common_drop_all_tables (GdaMetaStore *store)
 		"_view_column_usage",
 		"_domain_constraints",
 		"_parameters",
-		"_routine_columns"
+		"_routine_columns",
+		"_table_indexes",
+		"_index_column_usage"
 	};
 	gchar *view_names [] = {
 		"_all_types",
